#!/usr/bin/perl
# With some other Unix Os, first line may be
#!/usr/local/bin/perl
# With Apache for Windows and ActiverPerl, first line may be
#!c:/program files/activeperl/bin/perl
#-Description-------------------------------------------
# Change a log file to make reverse DNS lookup on IPAdresses
# Merge several log files into one
# This tool is part of AWStats software
# See COPYING.TXT file about AWStats GNU General Public License.
#-------------------------------------------------------
#use diagnostics;
#use strict;


#-------------------------------------------------------
# Defines
#-------------------------------------------------------

# ---------- Init variables (Variable $TmpHashxxx are not initialized) --------
($LogFile)=();
# ---------- Init arrays --------
@wordlist = ();
# ---------- Init hash arrays --------
%monthnum = ();

$VERSION="1.0 (build 1)";
$Lang="en";
$NbOfLinesForBenchmark=5000;

# These table is used to make fast reverse DNS lookup for particular IP adresses. You can add your own IP adresses resolutions.
%MyDNSTable = (
"256.256.256.1", "myworkstation1",
"256.256.256.2", "myworkstation2"
);



#-------------------------------------------------------
# Functions
#-------------------------------------------------------
sub error {
	print "Error.\n";
    die;
}

sub debug {
	my $level = $_[1] || 1;
	if ($Debug >= $level) { 
		my $debugstring = $_[0];
		if ($ENV{"GATEWAY_INTERFACE"} ne "") { $debugstring =~ s/^ /&nbsp&nbsp /; $debugstring .= "<br>"; }
		print "DEBUG $level - ".time." : $debugstring\n";
		}
	0;
}

sub SkipDNSLookup {
	foreach my $match (@SkipDNSLookupFor) { if ($_[0] =~ /$match/i) { return 1; } }
	0; # Not in @SkipDNSLookupFor
}


#------------------------------------------------------------------------------
# Function:     read config file
# Input:		$DIR $PROG $SiteToAnalyze
# Output:		Global variables
#------------------------------------------------------------------------------
sub Read_Config_File {
	$FileConfig="";
	my $logfilenb=1;
	my $Dir=$DIR; if (($Dir ne "") && (!($Dir =~ /\/$/)) && (!($Dir =~ /\\$/)) ) { $Dir .= "/"; }
	if (open(CONFIG,"$Dir$PROG.$SiteToAnalyze.conf")) { $FileConfig="$Dir$PROG.$SiteToAnalyze.conf"; $FileSuffix=".$SiteToAnalyze"; }
	if ($FileConfig eq "") { if (open(CONFIG,"$Dir$PROG.conf"))  { $FileConfig="$Dir$PROG.conf"; $FileSuffix=""; } }
	if ($FileConfig eq "") { error("Error: Couldn't open config file \"$PROG.$SiteToAnalyze.conf\" nor \"$PROG.conf\" : $!"); }
	&debug("Call to Read_Config_File [FileConfig=\"$FileConfig\"]");
	while (<CONFIG>) {
		chomp $_; s/\r//;
		$_ =~ s/#.*//;								# Remove comments
		$_ =~ tr/\t /  /s;							# Change all blanks into " "
		$_ =~ s/=//; @felter=split(//,$_);		# Change first "=" into "" to split
		$param=$felter[0]; $value=$felter[1];
		$value =~ s/^ *//; $value =~ s/ *$//;
		$value =~ s/^\"//; $value =~ s/\"$//;
		# Read main section
		if ($param =~ /^LogFile/) {
			$LogFile{$logfilenb}="$value";
			# Replace %YYYY %YY %MM %DD %HH with current value
			$LogFile{$logfilenb} =~ s/%YYYY/$nowyear/g;
			$LogFile{$logfilenb} =~ s/%YY/$nowsmallyear/g;
			$LogFile{$logfilenb} =~ s/%MM/$nowmonth/g;
			$LogFile{$logfilenb} =~ s/%DD/$nowday/g;
			$LogFile{$logfilenb} =~ s/%HH/$nowhour/g;
			$logfilenb++;
			next;
			}
		if ($param =~ /^LogFormat/)            { $LogFormat=$value; next; }
		if ($param =~ /^AllowToUpdateStatsFromBrowser/)	{ $AllowToUpdateStatsFromBrowser=$value; next; }
		if ($param =~ /^HostAliases/) {
			@felter=split(/\s+/,$value);
			$i=0; foreach $elem (@felter)      { $HostAliases[$i]=$elem; $i++; }
			next;
			}
		if ($param =~ /^SkipHosts/) {
			@felter=split(/\s+/,$value);
			$i=0; foreach $elem (@felter)      { $SkipHosts[$i]=$elem; $i++; }
			next;
			}
		if ($param =~ /^SkipFiles/) {
			@felter=split(/\s+/,$value);
			$i=0; foreach $elem (@felter)      { $SkipFiles[$i]=$elem; $i++; }
			next;
			}
		if ($param =~ /^OnlyFiles/) {
			@felter=split(/\s+/,$value);
			$i=0; foreach $elem (@felter)      { $OnlyFiles[$i]=$elem; $i++; }
			next;
			}
		if ($param =~ /^SkipDNSLookupFor/) {
			@felter=split(/\s+/,$value);
			$i=0; foreach $elem (@felter)      { $SkipDNSLookupFor[$i]=$elem; $i++; }
			next;
			}
		if ($param =~ /^DirData/)               { $DirData=$value; next; }
		if ($param =~ /^DirCgi/)                { $DirCgi=$value; next; }
		if ($param =~ /^DirIcons/)              { $DirIcons=$value; next; }
		if ($param =~ /^DNSLookup/)             { $DNSLookup=$value; next; }
		if ($param =~ /^PurgeLogFile/)          { $PurgeLogFile=$value; next; }
		if ($param =~ /^ArchiveLogRecords/)     { $ArchiveLogRecords=$value; next; }
		# Read optional section
		if ($param =~ /^Lang/)                  { $Lang=$value; next; }
		if ($param =~ /^DefaultFile/)           { $DefaultFile=$value; next; }
		if ($param =~ /^WarningMessages/)       { $WarningMessages=$value; next; }
		if ($param =~ /^ShowLinksOnUrl/)        { $ShowLinksOnUrl=$value; next; }
		if ($param =~ /^ShowFlagLinks/)         { $ShowFlagLinks=$value; next; }
		if ($param =~ /^HTMLEndSection/)        { $HTMLEndSection=$value; next; }
		if ($param =~ /^BarWidth/)              { $BarWidth=$value; next; }
		if ($param =~ /^BarHeight/)             { $BarHeight=$value; next; }
		if ($param =~ /^MaxNbOfDomain/)         { $MaxNbOfDomain=$value; next; }
		if ($param =~ /^MaxNbOfHostsShown/)     { $MaxNbOfHostsShown=$value; next; }
		if ($param =~ /^MinHitHost/)            { $MinHitHost=$value; next; }
		if ($param =~ /^MaxNbOfRobotShown/)     { $MaxNbOfRobotShown=$value; next; }
		if ($param =~ /^MinHitRobot/)           { $MinHitRobot=$value; next; }
		if ($param =~ /^MaxNbOfPageShown/)      { $MaxNbOfPageShown=$value; next; }
		if ($param =~ /^MinHitFile/)            { $MinHitFile=$value; next; }
		if ($param =~ /^MaxNbOfRefererShown/)   { $MaxNbOfRefererShown=$value; next; }
		if ($param =~ /^MinHitRefer/)           { $MinHitRefer=$value; next; }
		if ($param =~ /^MaxNbOfKeywordsShown/)  { $MaxNbOfKeywordsShown=$value; next; }
		if ($param =~ /^MinHitKeyword/)         { $MinHitKeyword=$value; next; }
		if ($param =~ /^SplitSearchString/)     { $SplitSearchString=$value; next; }
		if ($param =~ /^Logo/)                  { $Logo=$value; next; }
		if ($param =~ /^color_Background/)      { $color_Background=$value; next; }
		if ($param =~ /^color_TableTitle/)      { $color_TableTitle=$value; next; }
		if ($param =~ /^color_TableBGTitle/)    { $color_TableBGTitle=$value; next; }
		if ($param =~ /^color_TableRowTitle/)   { $color_TableRowTitle=$value; next; }
		if ($param =~ /^color_TableBGRowTitle/) { $color_TableBGRowTitle=$value; next; }
		if ($param =~ /^color_TableBG/)         { $color_TableBG=$value; next; }
		if ($param =~ /^color_TableBorder/)     { $color_TableBorder=$value; next; }
		if ($param =~ /^color_link/)            { $color_link=$value; next; }
		if ($param =~ /^color_hover/)           { $color_hover=$value; next; }
		if ($param =~ /^color_text/)            { $color_text=$value; next; }
		if ($param =~ /^color_titletext/)       { $color_titletext=$value; next; }
		if ($param =~ /^color_v/)               { $color_v=$value; next; }
		if ($param =~ /^color_w/)               { $color_w=$value; next; }
		if ($param =~ /^color_p/)               { $color_p=$value; next; }
		if ($param =~ /^color_h/)               { $color_h=$value; next; }
		if ($param =~ /^color_k/)               { $color_k=$value; next; }
		if ($param =~ /^color_s/)               { $color_s=$value; next; }
	}
	close CONFIG;
}


#------------------------------------------------------------------------------
# Function:     Get the messages for a specified language
# Parameter:	Language id
# Input:		None
# Output:		$message table
#------------------------------------------------------------------------------
sub Read_Language_Data {
	my $FileLang="";
	my $Dir=$DIR; if (($Dir ne "") && (!($Dir =~ /\/$/)) && (!($Dir =~ /\\$/)) ) { $Dir .= "/"; }
	if (open(LANG,"${Dir}lang/awstats-$_[0].txt")) { $FileLang="${Dir}lang/awstats-$_[0].txt"; }
	else { if (open(LANG,"${Dir}lang/awstats-en.txt")) { $FileLang="${Dir}lang/awstats-en.txt"; } }		# If file not found, we try english
	&debug("Call to Read_Language_Data [FileLang=\"$FileLang\"]");
	if ($FileLang ne "") {
		$i = 0;
		while (<LANG>) {
			chomp $_; s/\r//;
			if ($_ =~ /^PageCode/i) {
				$_ =~ s/^PageCode=//i;
				$_ =~ s/#.*//;								# Remove comments
				$_ =~ tr/\t /  /s;							# Change all blanks into " "
				$_ =~ s/^ *//; $_ =~ s/ *$//;
				$_ =~ s/^\"//; $_ =~ s/\"$//;
				$PageCode = $_;
			}
			if ($_ =~ /^message/i) {
				$_ =~ s/^message\d+=//i;
				$_ =~ s/#.*//;								# Remove comments
				$_ =~ tr/\t /  /s;							# Change all blanks into " "
				$_ =~ s/^ *//; $_ =~ s/ *$//;
				$_ =~ s/^\"//; $_ =~ s/\"$//;
				$message[$i] = $_;
				$i++;
			}
		}
	}
	close(LANG);
}



#--------------------------------------------------------------------
# Input: All lobal variables
# Ouput: Change on some global variables
#--------------------------------------------------------------------
sub Check_Config {
	&debug("Call to Check_Config");
	# Main section
	if ($LogFormat =~ /^[\d]$/ && $LogFormat !~ /[1-3]/)  { error("Error: LogFormat parameter is wrong. Value is '$LogFormat' (should be 1 or 2 or a 'personalised AWtats log format string')"); }
	if ($DNSLookup !~ /[0-1]/)                            { error("Error: DNSLookup parameter is wrong. Value is '$DNSLookup' (should be 0 or 1)"); }
	# Optional section
	if ($AllowToUpdateStatsFromBrowser !~ /[0-1]/) { $AllowToUpdateStatsFromBrowser=1; }	# For compatibility, is 1 if not defined
	if ($PurgeLogFile !~ /[0-1]/)                { $PurgeLogFile=0; }
	if ($ArchiveLogRecords !~ /[0-1]/)           { $ArchiveLogRecords=1; }
	if ($DefaultFile eq "")                      { $DefaultFile="index.html"; }
	if ($WarningMessages !~ /[0-1]/)             { $WarningMessages=1; }
	if ($ShowLinksOnURL !~ /[0-1]/)              { $ShowLinksOnURL=1; }
	if ($ShowFlagLinks !~ /[0-1]/)               { $ShowFlagLinks=1; }
	if ($BarWidth !~ /^[\d][\d]*/)               { $BarWidth=260; }
	if ($BarHeight !~ /^[\d][\d]*/)              { $BarHeight=180; }
	if ($MaxNbOfDomain !~ /^[\d][\d]*/)          { $MaxNbOfDomain=25; }
	if ($MaxNbOfHostsShown !~ /^[\d][\d]*/)      { $MaxNbOfHostsShown=25; }
	if ($MinHitHost !~ /^[\d][\d]*/)             { $MinHitHost=1; }
	if ($MaxNbOfRobotShown !~ /^[\d][\d]*/)      { $MaxNbOfRobotShown=25; }
	if ($MinHitRobot !~ /^[\d][\d]*/)            { $MinHitRobot=1; }
	if ($MaxNbOfPageShown !~ /^[\d][\d]*/)       { $MaxNbOfPageShown=25; }
	if ($MinHitFile !~ /^[\d][\d]*/)             { $MinHitFile=1; }
	if ($MaxNbOfRefererShown !~ /^[\d][\d]*/)    { $MaxNbOfRefererShown=25; }
	if ($MinHitRefer !~ /^[\d][\d]*/)            { $MinHitRefer=1; }
	if ($MaxNbOfKeywordsShown !~ /^[\d][\d]*/)   { $MaxNbOfKeywordsShown=25; }
	if ($MinHitKeyword !~ /^[\d][\d]*/)          { $MinHitKeyword=1; }
	if ($SplitSearchString !~ /[0-1]/)           { $SplitSearchString=0; }
	if ($Logo eq "")                             { $Logo="awstats_logo1.png"; }
	$color_Background =~ s/#//g; if ($color_Background !~ /^[0-9|A-Z][0-9|A-Z]*$/i)           { $color_Background="FFFFFF";	}
	$color_TableBGTitle =~ s/#//g; if ($color_TableBGTitle !~ /^[0-9|A-Z][0-9|A-Z]*$/i)       { $color_TableBGTitle="CCCCDD"; }
	$color_TableTitle =~ s/#//g; if ($color_TableTitle !~ /^[0-9|A-Z][0-9|A-Z]*$/i)           { $color_TableTitle="000000"; }
	$color_TableBG =~ s/#//g; if ($color_TableBG !~ /^[0-9|A-Z][0-9|A-Z]*$/i)                 { $color_TableBG="CCCCDD"; }
	$color_TableRowTitle =~ s/#//g; if ($color_TableRowTitle !~ /^[0-9|A-Z][0-9|A-Z]*$/i)     { $color_TableRowTitle="FFFFFF"; }
	$color_TableBGRowTitle =~ s/#//g; if ($color_TableBGRowTitle !~ /^[0-9|A-Z][0-9|A-Z]*$/i) { $color_TableBGRowTitle="ECECEC"; }
	$color_TableBorder =~ s/#//g; if ($color_TableBorder !~ /^[0-9|A-Z][0-9|A-Z]*$/i)         { $color_TableBorder="ECECEC"; }
	$color_text =~ s/#//g; if ($color_text !~ /^[0-9|A-Z][0-9|A-Z]*$/i)           { $color_text="000000"; }
	$color_titletext =~ s/#//g; if ($color_titletext !~ /^[0-9|A-Z][0-9|A-Z]*$/i) { $color_titletext="000000"; }
	$color_link =~ s/#//g; if ($color_link !~ /^[0-9|A-Z][0-9|A-Z]*$/i)           { $color_link="0011BB"; }
	$color_hover =~ s/#//g; if ($color_hover !~ /^[0-9|A-Z][0-9|A-Z]*$/i)         { $color_hover="605040"; }
	$color_v =~ s/#//g; if ($color_v !~ /^[0-9|A-Z][0-9|A-Z]*$/i)                 { $color_v="F3F300"; }
	$color_w =~ s/#//g; if ($color_w !~ /^[0-9|A-Z][0-9|A-Z]*$/i)                 { $color_w="FF9933"; }
	$color_w =~ s/#//g; if ($color_p !~ /^[0-9|A-Z][0-9|A-Z]*$/i)                 { $color_p="4477DD"; }
	$color_h =~ s/#//g; if ($color_h !~ /^[0-9|A-Z][0-9|A-Z]*$/i)                 { $color_h="66F0FF"; }
	$color_k =~ s/#//g; if ($color_k !~ /^[0-9|A-Z][0-9|A-Z]*$/i)                 { $color_k="339944"; }
	$color_s =~ s/#//g; if ($color_s !~ /^[0-9|A-Z][0-9|A-Z]*$/i)                 { $color_s="8888DD"; }
	# Default value	for messages
	if ($message[0] eq "") { $message[0]="Unknown"; }
	if ($message[1] eq "") { $message[1]="Unknown (unresolved ip)"; }
	if ($message[2] eq "") { $message[2]="Others"; }
	if ($message[3] eq "") { $message[3]="View details"; }
	if ($message[4] eq "") { $message[4]="Day"; }
	if ($message[5] eq "") { $message[5]="Month"; }
	if ($message[6] eq "") { $message[6]="Year"; }
	if ($message[7] eq "") { $message[7]="Statistics of"; }
	if ($message[8] eq "") { $message[8]="First visit"; }
	if ($message[9] eq "") { $message[9]="Last visit"; }
	if ($message[10] eq "") { $message[10]="Number of visits"; }
	if ($message[11] eq "") { $message[11]="Unique visitors"; }
	if ($message[12] eq "") { $message[12]="Visit"; }
	if ($message[13] eq "") { $message[13]="Keyword"; }
	if ($message[14] eq "") { $message[14]="Search"; }
	if ($message[15] eq "") { $message[15]="Percent"; }
	if ($message[16] eq "") { $message[16]="Traffic"; }
	if ($message[17] eq "") { $message[17]="Domains/Countries"; }
	if ($message[18] eq "") { $message[18]="Visitors"; }
	if ($message[19] eq "") { $message[19]="Pages-URL"; }
	if ($message[20] eq "") { $message[20]="Hours (Server time)"; }
	if ($message[21] eq "") { $message[21]="Browsers"; }
	if ($message[22] eq "") { $message[22]="HTTP Errors"; }
	if ($message[23] eq "") { $message[23]="Referers"; }
	if ($message[24] eq "") { $message[24]="Search&nbsp;Keywords"; }
	if ($message[25] eq "") { $message[25]="Visitors domains/countries"; }
	if ($message[26] eq "") { $message[26]="hosts"; }
	if ($message[27] eq "") { $message[27]="pages"; }
	if ($message[28] eq "") { $message[28]="different pages"; }
	if ($message[29] eq "") { $message[29]="Access"; }
	if ($message[30] eq "") { $message[30]="Other words"; }
	if ($message[31] eq "") { $message[31]="Pages not found"; }
	if ($message[32] eq "") { $message[32]="HTTP Error codes"; }
	if ($message[33] eq "") { $message[33]="Netscape versions"; }
	if ($message[34] eq "") { $message[34]="IE versions"; }
	if ($message[35] eq "") { $message[35]="Last Update"; }
	if ($message[36] eq "") { $message[36]="Connect to site from"; }
	if ($message[37] eq "") { $message[37]="Origin"; }
	if ($message[38] eq "") { $message[38]="Direct address / Bookmarks"; }
	if ($message[39] eq "") { $message[39]="Origin not provided"; }
	if ($message[40] eq "") { $message[40]="Links from an Internet Search Engine"; }
	if ($message[41] eq "") { $message[41]="Links from an external page (other web sites except search engines)"; }
	if ($message[42] eq "") { $message[42]="Links from an internal page (other page on same site)"; }
	if ($message[43] eq "") { $message[43]="keywords used on search engines"; }
	if ($message[44] eq "") { $message[44]="Kb"; }
	if ($message[45] eq "") { $message[45]="Unresolved IP Address"; }
	if ($message[46] eq "") { $message[46]="Unknown OS (Referer field)"; }
	if ($message[47] eq "") { $message[47]="Required but not found URLs (HTTP code 404)"; }
	if ($message[48] eq "") { $message[48]="IP Address"; }
	if ($message[49] eq "") { $message[49]="Error&nbsp;Hits"; }
	if ($message[50] eq "") { $message[50]="Unknown browsers (Referer field)"; }
	if ($message[51] eq "") { $message[51]="Visiting robots"; }
	if ($message[52] eq "") { $message[52]="visits/visitor"; }
	if ($message[53] eq "") { $message[53]="Robots/Spiders visitors"; }
	if ($message[54] eq "") { $message[54]="Free realtime logfile analyzer for advanced web statistics"; }
	if ($message[55] eq "") { $message[55]="of"; }
	if ($message[56] eq "") { $message[56]="Pages"; }
	if ($message[57] eq "") { $message[57]="Hits"; }
	if ($message[58] eq "") { $message[58]="Versions"; }
	if ($message[59] eq "") { $message[59]="Operating Systems"; }
	if ($message[60] eq "") { $message[60]="Jan"; }
	if ($message[61] eq "") { $message[61]="Feb"; }
	if ($message[62] eq "") { $message[62]="Mar"; }
	if ($message[63] eq "") { $message[63]="Apr"; }
	if ($message[64] eq "") { $message[64]="May"; }
	if ($message[65] eq "") { $message[65]="Jun"; }
	if ($message[66] eq "") { $message[66]="Jul"; }
	if ($message[67] eq "") { $message[67]="Aug"; }
	if ($message[68] eq "") { $message[68]="Sep"; }
	if ($message[69] eq "") { $message[69]="Oct"; }
	if ($message[70] eq "") { $message[70]="Nov"; }
	if ($message[71] eq "") { $message[71]="Dec"; }
	if ($message[72] eq "") { $message[72]="Navigation"; }
	if ($message[73] eq "") { $message[73]="Day statistics"; }
	if ($message[74] eq "") { $message[74]="Update now"; }
	if ($message[75] eq "") { $message[75]="Bytes"; }
	if ($message[76] eq "") { $message[76]="Last update"; }
	if ($message[77] eq "") { $message[77]="Top"; }
	if ($message[78] eq "") { $message[78]="dd mmm yyyy - HH:MM"; }
}



#-------------------------------------------------------
# MAIN
#-------------------------------------------------------
if ($ARGV[0] eq "-h") { $SiteToAnalyze = $ARGV[1]; }	# Kept for backward compatibility but useless
$QueryString=""; for (0..@ARGV-1) { $QueryString .= "$ARGV[$_] "; }
$QueryString =~ s/<script.*$//i;						# This is to avoid 'Cross Site Scripting attacks'
if ($QueryString =~ /site=/) { $SiteToAnalyze=$QueryString; $SiteToAnalyze =~ s/.*site=//; $SiteToAnalyze =~ s/&.*//; $SiteToAnalyze =~ s/ .*//; }
$UpdateStats=1;	if ($QueryString =~ /update=0/i) { $UpdateStats=0; }	# Update by default when run from command line
if ($QueryString =~ /debug=/i) { $Debug=$QueryString; $Debug =~ s/.*debug=//; $Debug =~ s/&.*//; $Debug =~ s/ .*//; }
($DIR=$0) =~ s/([^\/\\]*)$//; ($PROG=$1) =~ s/\.([^\.]*)$//; $Extension=$1;
if ($SiteToAnalyze eq "") { $SiteToAnalyze = $ENV{"SERVER_NAME"}; }
$SiteToAnalyze =~ tr/A-Z/a-z/;
$SiteToAnalyzeWithoutwww = $SiteToAnalyze; $SiteToAnalyzeWithoutwww =~ s/www\.//;
if (($ENV{"GATEWAY_INTERFACE"} eq "") && ($SiteToAnalyze eq "")) {
	print "----- $PROG $VERSION (c) Laurent Destailleur -----\n";
	print "$PROG is a log file merger and fast reverse DNS resolver.\n";
	print "$PROG comes with ABSOLUTELY NO WARRANTY. It's a free software\n";
	print "distributed with a GNU General Public License (See COPYING.txt file).\n";
	print "\n";
	print "Syntax: $PROG.$Extension file1 ... filen [-dnslookup]\n";
	print "  This runs $PROG in command line to open one or several web server\n";
	print "  log files to merge them (sorted on date) and/or to make a reverse DNS lookup.\n";
	print "  The result log file is sent on standard output.\n";
	print "Option:\n";
	print "  -dnslookup  make a reverse DNS lookup on IP adresses (not done by default).\n";
	print "  -showsteps  to add benchmark informations every $NbOfLinesForBenchmark lines processed\n";
	print "\n";
	print "Now supports/detects:\n";
	print "  Automatic detection of log format\n";
	print "  Multithreaded reverse DNS lookup ($LookupPool parallel request)\n";
	print "  No need of extra Perl library\n";
	print "New versions and FAQ at http://awstats.sourceforge.net\n";
	exit 0;
}

# Get current time
$nowtime=time;
($nowsec,$nowmin,$nowhour,$nowday,$nowmonth,$nowyear) = localtime($nowtime);
if ($nowyear < 100) { $nowyear+=2000; } else { $nowyear+=1900; }
$nowsmallyear=$nowyear;$nowsmallyear =~ s/^..//;
if (++$nowmonth < 10) { $nowmonth = "0$nowmonth"; }
if ($nowday < 10) { $nowday = "0$nowday"; }
if ($nowhour < 10) { $nowhour = "0$nowhour"; }
if ($nowmin < 10) { $nowmin = "0$nowmin"; }
if ($nowsec < 10) { $nowsec = "0$nowsec"; }
# Get tomorrow time (will be used to discard some record with corrupted date (future date))
($tomorrowsec,$tomorrowmin,$tomorrowhour,$tomorrowday,$tomorrowmonth,$tomorrowyear) = localtime($nowtime+86400);
if ($tomorrowyear < 100) { $tomorrowyear+=2000; } else { $tomorrowyear+=1900; }
$tomorrowsmallyear=$tomorrowyear;$tomorrowsmallyear =~ s/^..//;
if (++$tomorrowmonth < 10) { $tomorrowmonth = "0$tomorrowmonth"; }
if ($tomorrowday < 10) { $tomorrowday = "0$tomorrowday"; }
if ($tomorrowhour < 10) { $tomorrowhour = "0$tomorrowhour"; }
if ($tomorrowmin < 10) { $tomorrowmin = "0$tomorrowmin"; }
if ($tomorrowsec < 10) { $tomorrowsec = "0$tomorrowsec"; }
$timetomorrow=$tomorrowyear.$tomorrowmonth.$tomorrowday.$tomorrowhour.$tomorrowmin.$tomorrowsec;	

# Read config file
&Read_Config_File;
if ($QueryString =~ /lang=/i) { $Lang=$QueryString; $Lang =~ s/.*lang=//i; $Lang =~ s/&.*//; $Lang =~ s/\s+//; }
if ($Lang eq "") { $Lang="en"; }

# Get the output strings
&Read_Language_Data($Lang);

# Check and correct bad parameters
&Check_Config;

# Init other parameters
if ($ENV{"GATEWAY_INTERFACE"} ne "") { $DirCgi=""; }
if (($DirCgi ne "") && !($DirCgi =~ /\/$/) && !($DirCgi =~ /\\$/)) { $DirCgi .= "/"; }
if ($DirData eq "" || $DirData eq ".") { $DirData=$DIR; }	# If not defined or choosed to "." value then DirData is current dir
if ($DirData eq "")  { $DirData="."; }						# If current dir not defined then we put it to "."
$DirData =~ s/\/$//;
if ($DNSLookup) { use Socket; }
$NewDNSLookup=$DNSLookup;
%monthlib =  ( "01","$message[60]","02","$message[61]","03","$message[62]","04","$message[63]","05","$message[64]","06","$message[65]","07","$message[66]","08","$message[67]","09","$message[68]","10","$message[69]","11","$message[70]","12","$message[71]" );
# monthnum must be in english because it's used to translate log date in apache log files which are always in english
%monthnum =  ( "Jan","01","jan","01","Feb","02","feb","02","Mar","03","mar","03","Apr","04","apr","04","May","05","may","05","Jun","06","jun","06","Jul","07","jul","07","Aug","08","aug","08","Sep","09","sep","09","Oct","10","oct","10","Nov","11","nov","11","Dec","12","dec","12" );

#------------------------------------------
# PROCESSING CURRENT LOG(s)
#------------------------------------------
&debug("Start of processing log file(s)");
%LogFileToDo=(); %NowNewLinePhase=(); %NbOfLinesRead=(); %NbOfLinesCorrupted=();
$NbOfNewLinesProcessed=0; $NbOfNewLinesCorrupted=0;
$logfilechoosed=0;
$starttime=time();

# Open all log files
foreach $logfilenb (keys %LogFile) {
	&debug("Open log file number $logfilenb: \"$LogFile{$logfilenb}\"");
	open("LOG$logfilenb","$LogFile{$logfilenb}") || error("Error: Couldn't open server log file \"$LogFile{$logfilenb}\" : $!");
	$LogFileToDo{$logfilenb}=$LogFile{$logfilenb};
}

while (1 == 1)
{
	# BEGIN Read new record (for each log file or only for log file with record just processed)
	#------------------------------------------------------------------------------------------
	foreach $logfilenb (keys %LogFileToDo) {
		if (($logfilechoosed == 0) || ($logfilechoosed == $logfilenb)) {
			&debug("Search new record in file number $logfilenb",3);
			# Read choosed log file until found a record with good date or reching end of file
			while (1 == 1) {
				my $LOG="LOG$logfilenb"; $_=<$LOG>;	# Read new line
				if (! $_) {							# No more records in log file number $logfilenb
					&debug(" No more records in file number $logfilenb",2);
					delete $LogFileToDo{$logfilenb};
					last; }											# We have all the new records for each other files, we stop here

				chomp $_; s/\r//;

				if (/^#/) { next; }									# Ignore comment lines (ISS writes such comments)
				if (/^!!/) { next; }								# Ignore comment lines (Webstar writes such comments)
				if (/^$/) { next; }									# Ignore blank lines (With ISS: happens sometimes, with Apache: possible when editing log file)

				$NbOfLinesRead{$logfilenb}++;

				# Check filters
				#----------------------------------------------------------------------
				# Split DD/Month/YYYY:HH:MM:SS or YYYY-MM-DD HH:MM:SS or MM/DD/YY\tHH:MM:SS
				$_ =~ tr/-\/ \t/::::/;
				if ($_ =~ /(\d\d\d\d):(\d\d):(\d\d):(\d\d):(\d\d):(\d\d)/) { $dateparts[0]{$logfilenb}=$1; $dateparts[1]{$logfilenb}=$2; $dateparts[2]{$logfilenb}=$3; $dateparts[3]{$logfilenb}=$4; $dateparts[4]{$logfilenb}=$5; }
				if ($_ =~ /(\d\d):(\d\d):(\d\d):(\d\d):(\d\d):(\d\d)/) { $dateparts[0]{$logfilenb}=$1; $dateparts[1]{$logfilenb}=$2; $dateparts[2]{$logfilenb}=$3; $dateparts[3]{$logfilenb}=$4; $dateparts[4]{$logfilenb}=$5; }
				if ($monthnum{$dateparts[1]{$logfilenb}}) { $dateparts[1]{$logfilenb}=$monthnum{$dateparts[1]{$logfilenb}}; }	# Change lib month in num month if necessary
				# Create $timeconnexion like YYYYMMDDHHMMSS and $dayconnexion like YYYYMMDD
		 		$timeconnexion{$logfilenb}=$dateparts[0]{$logfilenb}.$dateparts[1]{$logfilenb}.$dateparts[2]{$logfilenb}.$dateparts[3]{$logfilenb}.$dateparts[4]{$logfilenb}.$dateparts[5]{$logfilenb};

				&debug(" This is a not already processed record",3);
				last;
			}
		}
	}
	# END Read new lines for each log file. After this, following var are filled
	# $field[x]{$logfilenb}, $dateparts[0-5]{$logfilenb}, $timeconnexion{$logfilenb}, $dayconnexion{$logfilenb}
	
	# We choose wich record of wich log file to process
	$logfilechoosed=-1;
	my $timeref="99999999999999";
	foreach $logfilenb (keys %LogFileToDo) {
		if ($timeconnexion{$logfilenb} < $timeref) { $logfilechoosed=$logfilenb; $timeref=$timeconnexion{$logfilenb} }
	}
	if ($logfilechoosed <= 0) { last; }								# No more record to process
	&debug("Analyse record of file number $logfilechoosed",3);
			
	# Record is approved. We found a new line to process in file number $logfilechoosed
	#----------------------------------------------------------------------------------
	$NbOfNewLinesProcessed++;
	if (($QueryString =~ /showstep=1/i) && ($NbOfNewLinesProcessed % $NbOfLinesForBenchmark == 0)) { print "$NbOfNewLinesProcessed lines processed (".(time()-$starttime)." seconds, ".($NbOfNewLinesProcessed/(time()-$starttime))." lines/seconds)\n"; }

	# Analyze: IP-address
	#--------------------
	$_ =~ /[\d]+\.[\d]+\.[\d]+\.[\d]+/;
	$Host=$1;
	if ($Host ne "") {
		$new=$TmpHashDNSLookup{$Host};	# TmpHashDNSLookup is a temporary hash table to increase speed
		if (!$new) {					# if $new undefined, $Host not yet resolved
			&debug(" Start of reverse DNS lookup for $Host",4);
			if ($MyDNSTable{$Host}) {
				&debug(" End of reverse DNS lookup, found resolution of $Host in local MyDNSTable",4);
				$new = $MyDNSTable{$Host};
			}
			else {
				if (&SkipDNSLookup($Host)) {
					&debug(" (Skipping this DNS lookup at user request.)",4);
				}
				else {
					$new=gethostbyaddr(pack("C4",split(/\./,$Host)),AF_INET);	# This is very slow, may took 20 seconds
				}
				&debug(" End of reverse DNS lookup for $Host",4);
			}
			if ($new eq "") { $new="ip"; }
			$TmpHashDNSLookup{$Host}=$new;
		}
		# Here $Host is still xxx.xxx.xxx.xxx and $new is name or "ip" if reverse failed)
		if ($new ne "ip") { $_ =~ s/$Host/$new/; }
    }
	else {
		&debug(" No IP adresses found in this record.",3);
	}

	print $_;
	# End of processing all new records.
}
&debug("End of processing log file(s)");


# Close all log files
foreach $logfilenb (keys %LogFile) {
	&debug("Close log file number $logfilenb");
	close("LOG$logfilenb");
}


0;	# Do not remove this line
